
// --------------------------------------------------------------------------------

//******************************************************************************
// Battery notification callback handler.
//******************************************************************************
static void BatteryNotifyCallback(BLERemoteCharacteristic* pBLERemoteCharacteristic,uint8_t* pData,size_t length,bool isNotify) {
  // called whenever a battery notification is received
  Serial.println("Battery Notification Callback Event");
  Serial.print("Data length ");
  Serial.println(length);
  Serial.print("data: ");

  for (int i = 0; i < length; i++) {Serial.printf("%02X ", pData[i]);}
  Serial.println();
}

// --------------------------------------------------------------------------------

//******************************************************************************
// Validate the server has the correct name and services we are looking for.
// The server must have the HID service, the Battery Service is optional.
//******************************************************************************
bool connectToServer() {
  Serial.print("Forming a connection to ");
  Serial.println(myDevice->getAddress().toString().c_str());
  
  BLEClient*  pClient  = BLEDevice::createClient();
  Serial.println(" - Created client");

  pClient->setClientCallbacks(new MyClientCallback());

  // Connect to the remote BLE Server.
  pClient->connect(myDevice);  // if you pass BLEAdvertisedDevice instead of address, it will be recognized type of peer device address (public or private)
  Serial.println(" - Connected to server");

  // BLE servers may offer several services, each with unique characteristics
  // we can identify the type of service by using the service UUID
  
  // Obtain a reference to the service we are after in the remote BLE server.
  // this will return a pointer to the remote service if it has a matching service UUID
  BLERemoteService* pRemoteService = pClient->getService(serviceUUID);
  if (pRemoteService == nullptr) {
    Serial.print("Failed to find HID service UUID: ");
    Serial.println(serviceUUID.toString().c_str());
    pClient->disconnect();
    return false;
  }

  Serial.println(" - Found HID service");
  setupCharacteristics(pRemoteService, notifyCallback);
  pRemoteService = pClient->getService(BatteryServiceUUID);
  if (pRemoteService == nullptr) {
    Serial.print("Failed to find battery service UUID: ");
    Serial.println(serviceUUID.toString().c_str());
  } else {
    Serial.println(" - Found battery service");
    setupCharacteristics(pRemoteService, BatteryNotifyCallback);
  }

  connected = true;
  return true;
}

// --------------------------------------------------------------------------------

void HandleButtons(uint8_t* pData,size_t length) {
  // buttons function is called when a 4 byte number is received
  // clear the flags every time this handler is called
  RxByte &= 0b11001111;   // clear the CZ buttons
  uint8_t buttons;
    if (0x50 == pData[1]) {
      // A/B button report, wake the A/B button handler task
      R1_Data[R1_BTNAB] = pData[0];
      // if (HandleAB) {vTaskResume(HandleAB);}
      buttons = R1_Data[R1_BTNAB];
      // Serial.printf("A/B Buttons: %02X\n", buttons);
        
      if (buttons & R1_BUTTON_A) {
        // button A pressed or is being held down
        // Serial.println("Button A");
        RxByte |= 0b00110000;
        //===== add your code here =====
      }

      if (buttons & R1_BUTTON_B) {
        // button B pressed or is being held down
        // Serial.println("Button B");
        RxByte |= 0b00110000;
        //===== add your code here =====
      }
    } else {
      // C/D button report
      R1_Data[R1_BTNCD] = pData[0];
      // if (HandleCD) {vTaskResume(HandleCD);}
      buttons = R1_Data[R1_BTNCD];
      // Serial.printf("C/D Buttons: %02X\n", buttons);
      
      if (buttons & R1_BUTTON_C) {
        // button C pressed
        // Serial.println("Button C");
        RxByte |= 0b00110000;
        //===== add your code here =====
      }

      if (buttons & R1_BUTTON_D) {
        // button D pressed
        // Serial.println("Button D");
        RxByte |= 0b00110000;
        //===== add your code here =====
      }
    }
}

// --------------------------------------------------------------------------------

void HandleJS() {
  // Magicsee joystick function is called when a 4 byte number is received
  // then six signed 8 bit integers
  // Here we look at joystick buttons as well as the C and Z buttons
  uint8_t triggers;
  int8_t  x;
  int8_t  y;
  x = (int8_t)R1_Data[R1_JOYX];
  y = (int8_t)R1_Data[R1_JOYY];
  triggers = R1_Data[R1_TRIGGERS];
  
  // Serial.printf("Joystick X: %d, Y: %d Triggers: %02X\n", x, y, triggers);
  // Serial.print("t1 - t0: "); Serial.println(BLEt1);

  // Here we contruct a control byte in a similar way to ESP-NOW WiFimode
  RxByte &= 0b00000000;   // clear the joystick bits
  // construct RxByte from received data, for use in PID controller
  // 00000000
  // ||||||||
  // |||||||!--- B0 turn right bit
  // ||||||!---- B1 turn left bit
  // |||||!----- B2 go forward bit
  // ||||!------ B3 go backwards bit
  // |||!------- B4 Z button
  // ||!-------- B5 C button
  // |!--------- B6 not used
  // !---------- B7 not used
  // do this only when in self-balance or in TEST mode
  if ((MainMode == 1) || (SafeMode == 4)) {

    if (y < -JoyStickDeadZone) {
      // move forward JoyY+
      // Serial.println("Forward");
      RxByte |= 0b00000100;
    } else if (y > JoyStickDeadZone) {
      // move backward JoyY-
      // Serial.println("Backward");
      RxByte |= 0b00001000;
    }
        
    if (x > JoyStickDeadZone) {
      // turn right JoyX+
      // Serial.println("Turn Right");
      RxByte |= 0b00000001;
    } else if (x < -JoyStickDeadZone) {
      // turn left JoyX-
      // Serial.println("Turn Left");
      RxByte |= 0b00000010;
    }

    if (triggers & R1_LOW_TRIGGER) {
      // the lower trigger button is pressed
      // Z-button, if C button is not pressed
      // Serial.println("Low Trigger Pressed");
      RxByte |= 0b00010000;
    }

    if (triggers & R1_UPR_TRIGGER) {
      // the upper trigger button is pressed
      // C-button
      // note that pressing the C button hides the Z button
      // Serial.println("Upper Trigger Pressed");
      RxByte |= 0b00100000;
    }
  }
  // restart the joystick timer
  joyTimer = millis() + JOYTIMEOUT;
}

// --------------------------------------------------------------------------------

//******************************************************************************
// HID notification callback handler.
//******************************************************************************
static void notifyCallback(BLERemoteCharacteristic* pBLERemoteCharacteristic, uint8_t* pData, size_t length, bool isNotify) {
  // called whenever a characteristic changes on the HID device
  // uncomment this section if you want to see the notification
  // individual tasks report the data received
  BLEt1 = millis() - BLEt0; BLEt0 = millis();

  // Serial.print("Notify callback for characteristic ");

  // Serial.print(pBLERemoteCharacteristic->getUUID().toString().c_str());
  // Serial.print(" of data length ");
  // Serial.println(length);

  Serial.print("data: ");
  for (int i = 0; i < length; i++) {Serial.printf("%02X ", pData[i]);}
  Serial.println();
 
  // we are getting the two trigger buttons in the first byte, joyX & joyY in 2nd & 3rd bytes
  // A four byte report is the joystick/trigger buttons.
  // A two byte report is either the A/B buttons or the C/D buttons
  // Low nibble equal to 0x05 indicates A/B buttons.
  // A/B buttons auto-repeat if held. No other buttons do this.
  if (4 == length) {
    // copy data to R1_Data
    for (int i = R1_TRIGGERS; i < R1_BTNAB; i++) {R1_Data[i] = pData[i];}

    // call the joystick/trigger buttons function
    HandleJS();
  } else if (2 == length) {
    // 2 bytes of received data
    HandleButtons(pData, length);
  }
}

// --------------------------------------------------------------------------------

//******************************************************************************
// Connect to a service, register for notifications from Report Characteristics.
//******************************************************************************
bool setupCharacteristics(BLERemoteService* pRemoteService, NotifyCallback pNotifyCallback) {
  // Get all the characteristics of the service using the handle as the key
  // we register characteristics that can report and give notifications
  bool canNotify = false;

  pmapbh = pRemoteService->getCharacteristicsByHandle();
  
  // only interested in report characteristics that have the notify capability
  for (itrbh = pmapbh->begin(); itrbh != pmapbh->end(); itrbh++) {
    BLEUUID x = itrbh->second->getUUID();
    Serial.print("Characteristic UUID: ");
    Serial.println(x.toString().c_str());
    // the uuid must match the report uuid

    if (ReportCharUUID.equals(itrbh->second->getUUID())) {
      // found a report characteristic
      Serial.println("Found a report characteristic");

      if (itrbh->second->canNotify()) {
        Serial.println("Can notify");
        // register for notifications from this characteristic
        itrbh->second->registerForNotify(pNotifyCallback);

        sprintf(bfr, "Callback registered for: Handle: 0x%08X, %d", itrbh->first, itrbh->first);
        Serial.println(bfr);
        canNotify = true;
      } else {
        Serial.println("No notification");
      }
    } else {
        // we do not want to display this once we are connected
        if (!connected) {
          sprintf(bfr, "Found characteristic UUID: %s\n", itrbh->second->getUUID().toString().c_str()); 
          Serial.println(bfr);
        }
    }
  } //  for
  return canNotify;
}

// --------------------------------------------------------------------------------

