
/* SSD1306Wire.h functions, see https://github.com/ThingPulse/esp8266-oled-ssd1306

  disply.clear()        - clear the local pixel buffer
  disply.display()      - write the buffer to the display
  disply.displayOff()   - turn the display OFF
  disply.displayOn()    - turn the display ON
  display.drawCircle(int16_t x, int16_t y, int16_t radius)
  display.drawFastImage(int16_t x, int16_t y, int16_t width, int16_t height, const uint8_t *image)
  display.drawHorizontalLine(int16_t x, int16_t y, int16_t length)
  display.drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1)  - draw from x0k,y0 to x1,y1
  display.drawProgressBar(uint16_t x, uint16_t y, uint16_t width, uint16_t height, uint8_t progress)
  display.drawRect(int16_t x, int16_t y, int16_t width, int16_t height)
  display.drawString(int16_t x, int16_t y, String text)
  display.drawStringMaxWidth(int16_t x, int16_t y, int16_t maxLineWidth, String text)
  display.drawVerticalLine(int16_t x, int16_t y, int16_t length)
  display.drawXbm(int16_t x, int16_t y, int16_t width, int16_t height, const char* xbm)
  display.fillCircle(int16_t x, int16_t y, int16_t radius)
  display.fillRect(int16_t x, int16_t y, int16_t width, int16_t height)
  display.flipScreenVertically()   - flip the current screen upside down
  disply.init()         - initialise the display
  disply.invertDisplay()- inverted display mode
  disply.end()          - free memory used by the display
  disply.displayOn()    - turn the display ON
  display.mirrorScreen() - draw the screen mirrored
  disply.normalDisplay()- normal display mode
  disply.reconnect()    - connect again through I2C
  display.setBrightness(uint8_t)
  display.setColor(OLEDDISPLAY_COLOR color) - BLACK, WHITE, INVERSE
  display.setContrast(uint8_t contrast, uint8_t precharge = 241, uint8_t comdetect = 64)
  display.setFont(const uint8_t* fontData)
  display.setPixel(int16_t x, int16_t y)  - set a pixel at x,y
  display.setTextAlignment(OLEDDISPLAY_TEXT_ALIGNMENT textAlignment)  - TEXT_ALIGN_LEFT, TEXT_ALIGN_CENTER, TEXT_ALIGN_RIGHT, TEXT_ALIGN_CENTER_BOTH

  W = display.getStringWidth(const char* text, uint16_t length)
  W = display.getStringWidth(String text)
  
  Display size is 128 x 64 (w x h) pixels
  Default fonts are ArialMT at 10, 16 & 24 pixels high
  Text options include:
    2 x 24, + 1 x 10 pnt lines
    1 x 24, + 2 x 16 pnt lines
    1 x 24, + 3 x 10 pnt lines
    4 x 16 pnt lines, no line spaces, decending characters will merge
    3 x 16, + 1 x 10 pnt lines
    6 x 10 pnt lines, no line spaces, decending characters will merge
*/

// --------------------------------------------------------------------------------

void Display_40ms() {
  // called every 40ms from the main loop if DispMon == true
  // if DispDel > 0 then we wait for DispDel cycles to complete before update
  // if (DispNOP) {return;}  // this function is aborted, no I2C activity
  
  if (DispDel > 0) {DispDel--; return;} // don't update display if DispDel > 0
  if (DispClr) {Display_clear(); DispClr = false; return;}
  if (DispDisp) {Display_display(); DispDisp = false; DispClr = true; return;}

  // We refresh individual displays based on a counter, DispCnt
  // Set DispCnt -ve to prevent the display update process
  
  if (DispCnt > 0) {DispCnt--;}
  else {
    // normal screens, display depends on MainMode values
    if (MainMode) {
      // display the following in TEST mode
      switch(DispMode) {
        // case  0: Display_Battery(); break;
        case -1: Display_Control(); break;
        case  0: Display_Battery(); break;
        case  1: Display_Status(); break;
        case  2: Display_SafeMode(); break;
        case  3: Display_PtchPID(); break;
        case  4: Display_DeadBand_Brake(); break;
        case  5: Display_OpMode(); break;
        case  6: Display_MPU_Levels(); break;
        case  7: Display_MPU_Acc(); break;
        case  8: Display_MPU_Gyro(); break;
        case  9: Display_MPU_Acc_Ang(); break;
        case 10: Display_MPU_Gyr_Ang(); break;
        case 11: Display_Range(); break;
        case 12: Display_Motor_Test(); break;
        case 13: Display_Motor_Stats(); break;
        case 14: Display_WiFiData(); break;
        case 15: Display_BLEData(); break;
        case 16: Display_GunFire(); break;
        case 17: Display_Drive(); break;
        default:  // catch-all
          DispMode = 0; break;
      }
      DM_Min = -1; DM_Max = 17;
    } else {
      // display the following in DEMO mode
      switch(DispMode) {
        case -1: Display_Control(); break;
        case  0: Display_Battery(); break;
        case  1: Display_Status(); break;
        case  2: Display_SafeMode(); break;
        case  3: Display_PtchPID(); break;
        case  4: Display_DeadBand_Brake(); break;
        case  5: Display_Filter(); break;
        case  6: Display_WiFiData(); break;
        case  7: Display_BLEData(); break;
        case  8: Display_Drive(); break;
        case  9: Display_BackAway(); break;
        case 10: Display_Range(); break;
        default:  // catch-all
          DispMode = 0; break;
      }
      DM_Min = -1; DM_Max = 10;
    }
  }
}

// --------------------------------------------------------------------------------

void Display_BackAway() {
  // display BackAway task data
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //  1111111111   2222222222    333333333   4444444444   5555555555
  Display_setFont(10);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 1, "Back Away");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,14, "Task:");
  Display_drawString(62,28, "Range:");
  Display_drawString(62,40, "Cnt:");
  Display_drawString(62,53, "Time:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(65,14, String(BackAwayTask));
  Display_drawString(65,28, String(Range));
  Display_drawString(65,40, String((Lft_Step_Cnt + Rht_Step_Cnt)/2));
  // Calculate time, based on 20ms counts, 50 counts per second
  Any$ = String(BackAwayTime/50) + ":"; int16_t zMod = BackAwayTime % 50;
  Any$ += String(zMod); Display_drawString(65,53, String(Any$));

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 4; // display for 160ms
}

// --------------------------------------------------------------------------------

void Display_Battery() {
  // normal screen to be displayed
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //  1111111111111111    2222222222222222        3333333333333333
  //                           4444444444444444
  Display_setFont(16); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 1, "Battery");
  if (BatAvg > BatCritical) {
    // battery powered mode
    if (BatVfp > 9.99) {Display_drawString(64, 21, String(BatVfp,1) + "v");}
    else {Display_drawString(64,21, String(BatVfp,2) + "v");}

    Display_setTextAlignment(TEXT_ALIGN_LEFT);
    Display_setFont(10);
    Display_drawString( 2,26, String(BatAvg));
    Display_setTextAlignment(TEXT_ALIGN_CENTER);
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    Display_drawString(125,26, String(BatPc) + "%");
    // display the time
    Display_setTextAlignment(TEXT_ALIGN_LEFT);
    Display_drawString( 2,45, getHmsTime(millis()));
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    Display_drawString(125,45, BatTime$);
  } else {
    // USB powered mode
    Display_setTextAlignment(TEXT_ALIGN_CENTER);
    Display_drawString(64,24, "USB Pwr");
    // display the time
    Display_drawString(64,45, getHmsTime(millis()));
  }
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 25; // update every 1s
}

// --------------------------------------------------------------------------------

void Display_BatWarning() {
  // battery <= 20% warning
  // we need a timer here to prevent this being called too often
  if ((millis() - DispBatWarn) < 30000)  {return;} // only action after 30 sec

  DispBatWarn = millis();  // reset the 30 sec timer
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //                1111111111111111    2222222222222222
  Display_setFont(16); Display_setColor(RED);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64,15, "Battery Low");
  Display_drawString(64,35, "< 20%");
  Display_drawRect(0,0,128,64);   // draw box outline

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 50; // hold off display update for 2sec
}

// --------------------------------------------------------------------------------

void Display_BLEData() {
  // display BLE data
  // this can only be viewed over USB, not WiFi
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111     2222222222     3333333333     4444444444
  Display_setFont(10);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "BLE Data");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,17, "Connected:");
  Display_drawString(62,32, "RxByte:");
  Display_drawString(62,47, "WiFiRxCnt:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Any$ = "False"; if (connected) {Any$ = "True";}
  Display_drawString(66,17, Any$);
  Display_drawString(66,32, getBinary(RxByte));
  Display_drawString(66,47, String(joyTimer));

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 4; // display for 160ms
}

// --------------------------------------------------------------------------------

void Display_Control() {
  // display a control screen, with options
  DM_Control = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111   2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "Control");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,15, "MainMode");
  Display_drawString(62,27, "Reset All");
  Display_drawString(62,39, "IsrT1");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(66,39, String(IsrT1));
  Display_setColor(BLUE);
  if (MainMode == 0) {Display_drawString(66,15, "Demo");}
  else {Display_drawString(66,15, "Test");}
  Display_drawString(66,27, ">Now");  // this is for Rest All


  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_DeadBand_Brake() {
  // display PID deadband and brake variables
  DM_DbBrake = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111  2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString( 63,2, "Db & Brake");
  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString( 62,14, "PIDop Db:");
  Display_drawString( 62,26, "Db Angle:");
  Display_drawString( 62,38, "PIDop Brake:");
  Display_drawString( 62,50, "Brake:");
  Display_drawString(124,50, String(1.0/Ptch_Brake,1));

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(64,26, String(Ptch_Db/Ptch_p_gain,2));

  Display_setColor(BLUE);
  Any$ = ""; if (Ptch_Db < 10.000) {Any$ = "0";}
  Display_drawString(64,14, Any$ + String(Ptch_Db,1));
  Any$ = ""; if (Ptch_OpBrake < 10.000) {Any$ = "0";}
  Display_drawString(64,38, Any$ + String(Ptch_OpBrake,2));
  Display_drawString(64,50, String(Ptch_Brake,3));

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 6; // update every 240ms
}

// --------------------------------------------------------------------------------

void Display_Drive() {
  // display a Drive screen, with options
  DM_Drive = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111  2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "Drive " + String(Gear));

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(32,14, "Turn:"); Display_drawString(96,14, "Spd:");
  Display_drawString(32,26, "Inc:"); Display_drawString(96,26, "Max:");
  Display_drawString(32,38, "PchSp:"); Display_drawString(96,38, "PIDop:");
  Display_drawString(32,50, "SpTgt:"); Display_drawString(96,50, "DrvMax:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(34,14, String(Turning)); Display_drawString(98,14, String(TurnSpd));
  Display_drawString(34,26, String(TurnSpdInc)); Display_drawString(98,26, String(TurnSpdMax));
  Display_drawString(34,38, String(PtchSp)); Display_drawString(98,38, String(Ptch_Op));

  Display_setColor(BLUE);
  Display_drawString(34,50, String(PtchSpTgt,1));
  int zX = 98; if (Drive_Max < 100.0) {zX = 103;}  if (Drive_Max < 10.0) {zX = 106;}
  Display_drawString(zX,50, String(Drive_Max,0));

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_Filter() {
  // display low pass filter control
  DM_Filter = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111     2222222222     3333333333
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "LP Filter");
  Display_drawString(64,17, "Vn=(LpFc* Vn)+(1-LpFc)*Vp");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,32, "LpFc:");

  Display_setColor(BLUE);
  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(66,32, String(LpFc,3));

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_GunFire() {
  // display Gun Fire controls
  DM_GunFire = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111     2222222222     3333333333
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "Gun Fire");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,17, "Fire:");
  Display_drawString(62,32, "GunFire:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(66,32, String(GunFire));
  Display_setColor(BLUE);
  Display_drawString(66,17, "Test");

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  if (GunFire > 0) {DispDel = 3;} else {DispDel = 5;}      // shorter delay when firing
}

// --------------------------------------------------------------------------------

void Display_Mirrored() {
  // Display robot type when DispMon is first connected
  // we clear current flags so that this is displayed immediately
  DispClr = false; DispDisp = false; DispCnt = 1;
  
  PrintTx+= "\n";       // terminate any half-sent command
  Display_clear();      // remove current display content
  if (Brightness != 255) {Display_setBrightness(255);}  // display may be in sleep mode
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  // 111111111111111111111111 2222222222222222           3333333333
  Display_setFont(24); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 0, "MandalBot");
  Display_setFont(10);
  Display_drawString(64,25, "BLE vR0");
  Display_drawString(64,51, "Display Mirror");

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();    // display immediately
  DispDel = 50;         // wait 2s before displaying next count
  DispMode = DispNext;  // drop back to the default battery display
}

// --------------------------------------------------------------------------------

void Display_Motor_Stats() {
  // Display motor variables
  DM_Motor_Stat = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111   2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(63, 2, "Motor Stats   ");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(30,15, "Thrttl:"); Display_drawString(94,15, "Thrttl:");
  Display_drawString(30,27, "L.Cnt:"); Display_drawString(94,27, "R.Cnt:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(32,15, String(Lft_Motor_Throttle)); Display_drawString(96,15, String(Rht_Motor_Throttle));
  Display_setColor(BLUE);
  // Any$ = String(Lft_Step_Cnt); Display_drawString(37,27, Any$);
  // Any$ = String(Lft_Step_Cnt); Display_drawString(98,27, Any$);
  Display_drawString(32,27, String(Lft_Step_Cnt));
  Display_drawString(96,27, String(Rht_Step_Cnt));

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

void Display_Motor_Test() {
  // display a Motor Test screen, with options
  DM_Motor_Test = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111   2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "Motor Test   ");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(30,15, "Lft:"); Display_drawString(94,15, "Rht:");
  Display_drawString(30,27, "Prd:"); Display_drawString(94,27, "Prd:");
  Display_drawString(30,39, "Dir:"); Display_drawString(94,39, "Dir:");
  Display_drawString(30,51, "M210:"); Display_drawString(94,51, "M210:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_setColor(BLUE);
  if (Motor_Test) {Any$ = "ON";} else {Any$ = "OFF";}
  Display_drawString(85, 2, Any$);

  if (Motor_Test) {Display_setColor(BLUE);} else {Display_setColor(WHITE);}
  if (Lft_En) {Any$ = "OFF";} else {Any$ = "ON";} Display_drawString(34,15, Any$);
  if (Rht_En) {Any$ = "OFF";} else {Any$ = "ON";} Display_drawString(98,15, Any$);
  if (Lft_Motor_Throttle > 0) {Display_drawString(34,27, String(Lft_Motor_Throttle));}
  else {Display_drawString(34,27, "Off");}
  if (Rht_Motor_Throttle > 0) {Display_drawString(98,27, String(Rht_Motor_Throttle));}
  else {Display_drawString(98,27, "Off");}
  if (Lft_Dir) {Any$ = "Hi";} else {Any$ = "Lo";} Display_drawString(34,39, Any$);
  if (Rht_Dir) {Any$ = "Hi";} else {Any$ = "Lo";} Display_drawString(98,39, Any$);
  Display_drawString(34,51, GetM210(M210)); Display_drawString(98,51, GetM210(M210));

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 8; // update every 320ms
}

// --------------------------------------------------------------------------------

void Display_MPU_Acc() {
  // display MPU6050 accelerometer information
  DM_MPU_Acc = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111      2222222222      3333333333      4444444444
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, "MPU Acc.");

  Display_setTextAlignment(TEXT_ALIGN_LEFT); Display_setColor(BLUE);
  if (AccOffEn) {Display_drawString(88, 3, "-");}
           else {Display_drawString(86, 3, "R");}

  if (AccOffEn) {Display_setColor(WHITE);} else {Display_setColor(YELLOW);}
  Display_drawString( 2,19, "AcX");
  if (AccOffEn) {Display_drawString(26,19, String(AcX));} else {Display_drawString(26,19, String(AcX + AccOffX));}
  if (MainMode == 1) {
    Display_drawString(70,19, "AvX");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (AccOffEn) {Display_drawString(125,19, String(AcXAvgL));} else {Display_drawString(125,19, String(AcXAvgL + AccOffX));}
  }

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString( 2,35, "AcY");
  if (AccOffEn) {Display_drawString(26,35, String(AcY));} else {Display_drawString(26,35, String(AcY + AccOffY));}
  if (MainMode == 1) {
    Display_drawString(70,35, "AvY");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (AccOffEn) {Display_drawString(125, 35, String(AcYAvgL));} else {Display_drawString(125, 35, String(AcYAvgL + AccOffY));}
  }

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString( 2,51, "AcZ");
  if (AccOffEn) {Display_drawString(26,51, String(AcZ));} else {Display_drawString(26,51, String(AcZ + AccOffZ));}
  if (MainMode == 1) {
    Display_drawString(70,51, "AvZ");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (AccOffEn) {Display_drawString(125,51, String(AcZAvgL));} else {Display_drawString(125,51, String(AcZAvgL + AccOffZ));}
  }

  if (MainMode) {Display_setColor(WHITE); Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MPU_Acc_Ang() {
  // displays the MPU angles
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111      2222222222      3333333333      4444444444
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, "MPU Acc Ang.");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,19, "Pitch");
  Display_drawString(62,35, "Roll");
  Display_drawString(62,51, "Yaw");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(66,19, String(PtchAccAng) + Deg);
  Display_drawString(66,35, String(RollAccAng) + Deg);
  Display_drawString(66,51, String(YawAccAng) + Deg);

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MPU_AccRamps() {
  // display MPU6050 accelerometer ramp rate limits
  DM_MPU_AccRamps = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111      2222222222      3333333333      4444444444
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, "Acc Ramps");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,19, "AccRampX");
  Display_drawString(62,35, "AccRampY");
  Display_drawString(62,51, "AccRampZ");

  Display_setColor(BLUE);
  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  if (AccRampX > 0) {Display_drawString(66,19, String(getNum00(AccRampX)));} else {Display_drawString(66,19, "Off");}
  if (AccRampY > 0) {Display_drawString(66,35, String(getNum00(AccRampY)));} else {Display_drawString(66,35, "Off");}
  if (AccRampZ > 0) {Display_drawString(66,51, String(getNum00(AccRampZ)));} else {Display_drawString(66,51, "Off");}

  if (MainMode) {Display_setColor(WHITE); Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MPU_Gyro() {
  // display MPU6050 gyro information
  DM_MPU_Gyros = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111      2222222222      3333333333      4444444444
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, "MPU Gyros");

  Display_setTextAlignment(TEXT_ALIGN_LEFT); Display_setColor(BLUE);
  if (GyrOffEn) {Display_drawString(92, 3, "-");}
           else {Display_drawString(90, 3, "R");}

  if (GyrOffEn) {Display_setColor(WHITE);} else {Display_setColor(YELLOW);}
  Display_drawString( 2,19, "GyX");
  if (GyrOffEn) {Display_drawString(26,19, String(GyX));} else {Display_drawString(26,19, String(GyX + GyrOffX));}
  if (MainMode == 1) {
    Display_drawString(70,19, "GvX");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (GyrOffEn) {Display_drawString(125,19, String(GyXAvgL));} else {Display_drawString(125,19, String(GyXAvgL + GyrOffX));}
  }

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString( 2,35, "GyY");
  if (GyrOffEn) {Display_drawString(26,35, String(GyY));} else {Display_drawString(26,35, String(GyY + GyrOffY));}
  if (MainMode == 1) {
    Display_drawString(70,35, "GvY");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (GyrOffEn) {Display_drawString(125,35, String(GyYAvgL));} else {Display_drawString(125,35, String(GyYAvgL + GyrOffY));}
  }

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString( 2,51, "GyZ");
  if (GyrOffEn) {Display_drawString(26,51, String(GyZ));} else {Display_drawString(26,51, String(GyZ + GyrOffZ));}
  if (MainMode == 1) {
    Display_drawString(70,51, "GvZ");
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if (GyrOffEn) {Display_drawString(125,51, String(GyZAvgL));} else {Display_drawString(125,51, String(GyZAvgL + GyrOffZ));}
  }

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MPU_Gyr_Ang() {
  // displays the MPU combined angles
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111      2222222222      3333333333      4444444444
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, "MPU Gyr Ang.");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,19, "Pitch");
  Display_drawString(62,35, "Roll");
  Display_drawString(62,51, "Yaw");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(66,19, String(PtchGyrAng) + Deg);
  Display_drawString(66,35, String(RollGyrAng) + Deg);
  Display_drawString(66,51, String(YawGyrAng) + Deg);

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MPU_Levels() {
  // display MPU6050 Pitch and Roll angles like spirit level
  Display_clear();
  Display_setFont(10);

  if ((fabs(PtchGyrAng) <= 2.00) && (fabs(RollGyrAng) <= 2.00)) {
    // angles are <= 2.0°
    // Y64 vertical text spacing, from Y=0 to Y=63
    // 0123456789012345678901234567890123456789012345678901234567890123
    // 1111111111                 2222222222
    Display_setTextAlignment(TEXT_ALIGN_LEFT);
    if (PtchGyrAng > 0.00) {
      Display_drawString(68, 0, String(PtchGyrAng,1) + Deg);
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(60, 0, "B");
    } else {
      Display_drawString(68,53, String(PtchGyrAng,1) + Deg);
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(60,53, "F");
    }
    Display_setTextAlignment(TEXT_ALIGN_LEFT);
    if (RollGyrAng > 0.00) {
      Display_drawString(100,26, String(RollGyrAng,1) + Deg);
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(28,26, "L");
    } else {
      Display_drawString(100,26, "R");
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(28,26, String(RollGyrAng,1) + Deg);
    }
    Display_drawLine(32,32,96,32);    // horizontal centre line
    DispCX = 62 + (16 * RollGyrAng);  // Roll bubble
    Display_drawRect(DispCX,30,4,4);  // horizontal bubble
    Display_drawLine(64,0,64,63);     // vertical centre line
    DispCY = 30 - (16 * PtchGyrAng);  // Pitch bubble
    Display_drawRect(62,DispCY,4,4);  // vertical bubble
  } else {
    // angles are >= 2.0°
    if (RollGyrAng >= 0.00) {
      // rolling left
      DispCX = 96; DispCY = 2 * fabs(RollGyrAng);
      Display_drawLine(DispCX,32 - DispCY,DispCX,32 + DispCY);  // horizontal wedge right
      Display_drawLine(32,32,DispCX,32 - DispCY);
      Display_drawLine(32,32,DispCX,32 + DispCY);
      Display_setTextAlignment(TEXT_ALIGN_LEFT);
      Display_drawString(DispCX + 4,26, String(RollGyrAng,1) + Deg);
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(28,26, "L");
    } else {
      // rolling right
      DispCX = 32; DispCY = 2 * fabs(RollGyrAng);
      Display_drawLine(DispCX,32 - DispCY,DispCX,32 + DispCY);  // horizontal wedge left
      Display_drawLine(96,32,DispCX,32 - DispCY);
      Display_drawLine(96,32,DispCX,32 + DispCY);
      Display_setTextAlignment(TEXT_ALIGN_RIGHT);
      Display_drawString(DispCX - 4,26, String(RollGyrAng,1) + Deg);
      Display_setTextAlignment(TEXT_ALIGN_LEFT);
      Display_drawString(100,26, "R");
    }
    if (PtchGyrAng >= 0.00) {
      // pitching backwards
      DispCX = 2 * fabs(PtchGyrAng); DispCY = 1; 
      Display_drawLine(64 - DispCX,DispCY,64 + DispCX,DispCY);  // vertical wedge up
      Display_drawLine(64,64,64 - DispCX,DispCY);
      Display_drawLine(64,64,64 + DispCX,DispCY);
      if (RollGyrAng < 0.00) {
        // rolling right
        Display_setTextAlignment(TEXT_ALIGN_LEFT);
        Display_drawString(70,53, String(PtchGyrAng,1) + Deg);
        Display_setTextAlignment(TEXT_ALIGN_RIGHT);
        Display_drawString(58,53, "B");
      } else {
        // rolling left
        Display_setTextAlignment(TEXT_ALIGN_RIGHT);
        Display_drawString(58,53, String(PtchGyrAng,1) + Deg);
        Display_setTextAlignment(TEXT_ALIGN_LEFT);
        Display_drawString(70,53, "B");
      }
    } else {
      // pitching forwards
      DispCX = 2 * fabs(PtchGyrAng); DispCY = 63; 
      Display_drawLine(64 - DispCX,DispCY,64 + DispCX,DispCY);  // vertical wedge down
      Display_drawLine(64,0,64 - DispCX,DispCY);
      Display_drawLine(64,0,64 + DispCX,DispCY);
      if (RollGyrAng < 0.00) {
        // rolling right
        Display_setTextAlignment(TEXT_ALIGN_LEFT);
        Display_drawString(70, 0, String(PtchGyrAng,1) + Deg);
        Display_setTextAlignment(TEXT_ALIGN_RIGHT);
        Display_drawString(58, 0, "F");
      } else {
        // rolling left
        Display_setTextAlignment(TEXT_ALIGN_RIGHT);
        Display_drawString(58, 0, String(PtchGyrAng,1) + Deg);
        Display_setTextAlignment(TEXT_ALIGN_LEFT);
        Display_drawString(70, 0, "F");
      }
    }
  }
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_MsCount() {
  // display the current millis() count
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //    1111111111        2222222222   3333333333
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 3, String(millis()));
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_OpMode() {
  // display OpMode variables
  DM_OpMode = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111  2222222222  3333333333
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(63,2, "Motor Calcs");
       if (OpMode == 0) {Any$ = "Hard Deadband";}
  else if (OpMode == 1) {Any$ = "Soft Deadband";}
  Display_drawString(63,26, "<" + Any$ + ">");
  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,14, "OpMode:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_setColor(BLUE);
  Display_drawString(64,14, String(OpMode));

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 10; // update every 400ms
}

// --------------------------------------------------------------------------------

void Display_PtchPID() {
  // display Pitch PID variables
  DM_PtchPID = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111  2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(78,2, "Pitch PID");
  Display_drawString(60,14, "P-gain");
  Display_drawString(60,26, "I-gain");
  Display_drawString(60,38, "D-gain");
  Display_drawString(60,50, "StartAng");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  if (PtchStartAng >= 0.0) {Display_drawString(64,50, "+");} else {Display_drawString(66,50, "-");}

  Display_setColor(BLUE);
  Any$ = ""; if (Ptch_p_gain < 10.00) {Any$ = "0";}
  Display_drawString(64,14, Any$ + String(Ptch_p_gain,2));
  if (Ptch_i_En) {
    Any$ = ""; if (Ptch_i_gain < 10.00) {Any$ = "0";}
    Display_drawString(64,26, Any$ + String(Ptch_i_gain,2));
  } else {Display_drawString(64,26, "---.---");}
  if (Ptch_d_En) {
    Any$ = ""; if (Ptch_d_gain < 10.00) {Any$ = "0";}
    Display_drawString(64,38, Any$ + String(Ptch_d_gain,2));}
  else {Display_drawString(64,38, "---.---");}
  Display_drawString(70,50, String(fabs(PtchStartAng),2) + Deg);

  Display_setColor(WHITE);
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 6; // update every 240ms
}

// --------------------------------------------------------------------------------

void Display_Range() {
  // display VL53L1X range measurement
  DM_Range = DispMode; // note Display Mode assignment
  String zSL = ""; String zSR = "";
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111111111      2222222222222222      3333333333333333
  //                                                   4444444444
  Display_setFont(16);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
       if (MainType == 2) {Display_drawString(64, 2, "Back Away");}
  else if (MainType == 3) {Display_drawString(64, 2, "Track Mode");}
  else if (MainType == 4) {Display_drawString(64, 2, Auton$);}
  else {
    if (MainMode) {Display_setColor(BLUE);}
    Display_drawString(64, 2, "Range");
    Display_setColor(WHITE);
  }
  if (RangeEn) {
         if (RangeUL) {zSL = "< "; zSR = " >";}
    else if (RangeLL) {zSL = "> "; zSR = " <";}
    Display_drawString(64,24, zSL + String(Range) + " mm" + zSR);
  } else {Display_drawString(65,24, "---");}
  Display_setFont(10);
  if (LTOFcalc) {
    Display_setTextAlignment(TEXT_ALIGN_LEFT);
    Display_drawString(0,50,String(LTOFfps) + " fps");
    
    Display_setTextAlignment(TEXT_ALIGN_RIGHT);
    if ((LTOFspd10 == 0) || RangeUL) {
      Display_drawString(127,50, "--- m/s");
    } else {
      Display_drawString(127,50, String(float(LTOFspd10)/10.0,1) + " m/s");
    }
  } else {
    if (RangeFps) {Display_drawString(64,50,String(LTOFfps) + " fps");}
    else {Display_drawString(64,50,Status$);}
    if (RangeEn && MainMode) {
      // Range is enabled in TEST mode
      Display_setTextAlignment(TEXT_ALIGN_LEFT);
      Display_drawString( 3,40, "CntM:");
      Display_drawString( 2,50, "Pk-Pk:");
      Display_drawString(90,50, "Rate:");
      Display_setColor(BLUE);
      Display_drawString( 30,40, String(RangeCntMax));
      Display_drawString( 30,50, String(RangePkPk));
      Display_drawString(113,50, String(RangeRate));
      Display_setColor(WHITE);
    }
  }
  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();  // display this immediately
  DispDel = 4;        // wait 160ms before displaying next count
}

// --------------------------------------------------------------------------------

void Display_SafeMode() {
  // display SafeMode + data
  DM_SafeMode = DispMode; // note Display Mode assignment
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111  2222222222  3333333333  4444444444  5555555555
  Display_setFont(10); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(63, 2, "SafeMode " + String(SafeMode));

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  if (SafeMode == 0) {Display_drawString(126, 2, String(SafeCnt));}
  Display_drawString(62,14, "PtchGyrAng:");
  Display_drawString(62,26, "SbSp:");
  Display_drawString(62,38, "PtchErr:");
  Display_drawString(126,38, String(PtchNan));
  Display_drawString(62,50, "Ptch_Op:");
  Display_drawString(126,50, String(Lft_Motor_Throttle));

  Display_setColor(BLUE);
    if (!Sb) {Display_drawString(126,26, "OFF");}
  else {Display_drawString(126,26, String(SbSpInc,4));}

  Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(64,14, String(PtchGyrAng) + Deg);
  Display_drawString(64,26, String(SbSp,3) + Deg);
  Display_drawString(64,38, String(PtchErr));
  Display_drawString(64,50, String(Ptch_Op));

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 5; // update every 200ms
}

// --------------------------------------------------------------------------------

void Display_Status() {
  // display status information
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111   2222222222   3333333333   4444444444
  Display_setFont(10);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(63,02, MyStatus);
  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,15, "Angle:");
  Display_drawString(62,28, "SM:");
  Display_drawString(62,41, "Batt:");
  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Display_drawString(64,15, String(PtchGyrAng,1) + Deg);
  Display_drawString(64,28, String(SafeMode));
  if (!USB) {Display_drawString(64,41, String(BatPc) + "%");}
  else {Display_drawString(64,41, "USB Pwr");}

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();  // display this immediately
  DispDel = 4;        // wait 160ms before displaying next count
}

// --------------------------------------------------------------------------------

void Display_Text_2x16(String zT0$,String zT1$,int16_t zD) {
  // display T0$ and T1$ as two lines of text, for zD x 40ms
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //             1111111111111111    2222222222222222
  Display_setFont(16); Display_setColor(WHITE);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64,12, zT0$);
  Display_drawString(64,32, zT1$);

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = zD; // display for zD x 40ms
}

// --------------------------------------------------------------------------------

void Display_Topple() {
  // display turn demans and acceleration limits
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //           1111111111111111           2222222222
  Display_setFont(16); Display_setColor(RED);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  switch (random(9)) {
    case 0: Display_drawString(64,10, "Ouch!"); break;
    case 1: Display_drawString(64,10, "Oh no!"); break;
    case 2: Display_drawString(64,10, "Oops!"); break;
    case 3: Display_drawString(64,10, "Crash!"); break;
    case 4: Display_drawString(64,10, "Aah!"); break;
    case 5: Display_drawString(64,10, "Wham!"); break;
    case 6: Display_drawString(64,10, "Smash!"); break;
    case 7: Display_drawString(64,10, "Ow!"); break;
    case 8: Display_drawString(64,10, "Wallop!"); break;
  }
  Display_setFont(10);
  switch (random(10)) {
    case 0: Display_drawString(64,37, "That hurt"); break;
    case 1: Display_drawString(64,37, "Hurt my wheels"); break;
    case 2: Display_drawString(64,37, "Try again"); break;
    case 3: Display_drawString(64,37, "Steady on"); break;
    case 4: Display_drawString(64,37, "Call a mechanic"); break;
    case 5: Display_drawString(64,37, "Stop it"); break;
    case 6: Display_drawString(64,37, "So painful"); break;
    case 7: Display_drawString(64,37, "Take more care"); break;
    case 8: Display_drawString(64,37, "Who's driving?"); break;
    case 9: Display_drawString(64,37, "Pick me up"); break;
  }
  Display_display();
  DispDel = 75; // display for 3 seconds
}

// --------------------------------------------------------------------------------

void Display_WiFiData() {
  // display WiFi data
  Display_clear();
  // Y64 vertical text spacing, from Y=0 to Y=63
  // 0123456789012345678901234567890123456789012345678901234567890123
  //   1111111111     2222222222     3333333333     4444444444
  Display_setFont(10);
  Display_setTextAlignment(TEXT_ALIGN_CENTER);
  Display_drawString(64, 2, "WiFi Data");

  Display_setTextAlignment(TEXT_ALIGN_RIGHT);
  Display_drawString(62,17, "Connected:");
  Display_drawString(62,32, "RxByte:");
  Display_drawString(62,47, "WiFiRxCnt:");

  Display_setTextAlignment(TEXT_ALIGN_LEFT);
  Any$ = "False"; if (WiFiConnected) {Any$ = "True";}
  Display_drawString(66,17, Any$);
  Display_drawString(66,32, getBinary(RxByte));
  Display_drawString(66,47, String(WiFiRxCnt));

  if (MainMode) {Display_drawRect(0,0,128,64);} // draw 'Test' box outline
  Display_display();
  DispDel = 4; // display for 160ms
}

// --------------------------------------------------------------------------------

