
// -------------------------------------------------------------------------------

void PID_readAcc() {
  // PID accelerometer readings from MPU6050
  AcXL = AcX; AcYL = AcY; AcZL = AcZ;   // track previous values
  Wire.beginTransmission(MPU_address);
  Wire.write(0x3B);                     // starting with register 0x3B (ACCEL_XOUT_H)
  I2CErr = Wire.endTransmission();      // End the transmission with the MPU.
  if (I2CErr != 0) {return;}            // abort on errors
  delayMicroseconds(I2Cdel);            // Allow MPU time to respond

  Wire.requestFrom(MPU_address,6);      // request 4 register read
  AcX = Wire.read()<<8|Wire.read();     // 0x3B (ACCEL_XOUT_H) & 0x3E (ACCEL_XOUT_L)
  AcY = Wire.read()<<8|Wire.read();     // 0x3D (ACCEL_YOUT_H) & 0x3E (ACCEL_YOUT_L)
  AcZ = Wire.read()<<8|Wire.read();     // 0x3F (ACCEL_ZOUT_H) & 0x40 (ACCEL_ZOUT_L)
  AcX -= AccOffX;  // note AcXOff should be zero initially in order to determine its value
  AcY -= AccOffY;  // note AcYOff should be zero initially in order to determine its value
  AcZ -= AccOffZ;  // note AcZOff should be zero initially in order to determine its value

  // //################################
  // // Averaging Noise Filters
  // //################################
  // This was tried, but not used
  // AcX = (AcX + AcXL)/2; AcY = (AcY + AcYL)/2; AcZ = (AcZ + AcZL)/2;
  
  // //################################
  // // rapid Roll overshoot
  // //################################
  // If rate of change of AccDif is too great, we limit value to change to AccRamp
  // This is to reject the acceleration associated with the stepper motors stepping
  if (AccRampX > 0) {
    AccDif = AcX - AcXL; if (fabs(AccDif) > AccRampX) {if (AcX > AcXL) {AcX = AcXL + AccRampX;} else {AcX = AcXL - AccRampX;}}
  }
  if (AccRampY > 0) {
    AccDif = AcY - AcYL; if (fabs(AccDif) > AccRampY) {if (AcY > AcYL) {AcY = AcYL + AccRampY;} else {AcY = AcYL - AccRampY;}}
  }
  if (AccRampZ > 0) {
    AccDif = AcZ - AcZL; if (fabs(AccDif) > AccRampZ) {if (AcZ > AcZL) {AcZ = AcZL + AccRampZ;} else {AcZ = AcZL - AccRampZ;}}
  }

  //##############################################
  // Averaging filters to reduce MPU random noise
  //##############################################
  // This was tried, but not used
  // AcXSum = AcXSum - AcXAvg + AcX; AcXAvg = AcXSum /AvCnt; AcX = AcXAvg;
  // AcYSum = AcYSum - AcYAvg + AcY; AcYAvg = AcYSum /AvCnt; AcY = AcYAvg;
  // AcZSum = AcZSum - AcZAvg + AcZ; AcZAvg = AcZSum /AvCnt; AcZ = AcZAvg;

  // Limit values to avoid trig maths function problems
  if (AcX > 8192) {AcX = 8192;} else if (AcX < -8192) {AcX = -8192;}
  if (AcY > 8192) {AcY = 8192;} else if (AcY < -8192) {AcY = -8192;}
  if (AcZ > 8192) {AcZ = 8192;} else if (AcZ < -8192) {AcZ = -8192;}
}

// -------------------------------------------------------------------------------

void PID_readGyr() {
  // Read all 3 gyroscopes, and apply drift offsets, determined from testing
  GyXL = GyX; GyYL = GyY; GyZL = GyZ;   // track previous values
  Wire.beginTransmission(MPU_address);
  Wire.write(0x43);                     // starting with register 0x43 (GYRO_XOUT_H)
  I2CErr = Wire.endTransmission();      // End the transmission with the MPU.
  if (I2CErr != 0) {return;}            // abort on errors
  delayMicroseconds(I2Cdel);                // Allow MPU time to respond

  Wire.requestFrom(MPU_address,6);      // request 6 register reads
  GyX = Wire.read()<<8|Wire.read();     // 0x43 (GYRO_XOUT_H) & 0x44 (GYRO_XOUT_L)
  GyY = Wire.read()<<8|Wire.read();     // 0x45 (GYRO_YOUT_H) & 0x46 (GYRO_YOUT_L)
  GyZ = Wire.read()<<8|Wire.read();     // 0x47 (GYRO_ZOUT_H) & 0x48 (GYRO_ZOUT_L)
  // get measurement timing
  GyTD = micros() - GyT;                // get the differential time between readings
  GyT = GyTD + GyT;                     // set the timer = micros(), for the next reading
  // apply calibration offsets to these raw values
  GyX -= GyrOffX;  // note GyrOffX should be zero initially in order to determin its value
  GyY -= GyrOffY;  // note GyrOffY should be zero initially in order to determin its value
  GyZ -= GyrOffZ;  // note GyrOffZ should be zero initially in order to determin its value

  // //################################
  // // Averaging Noise Filters
  // //################################
  // This was tried, but not used
  // GyX = (GyX + GyXL)/2; GyY = (GyY + GyYL)/2; GyZ = (GyZ + GyZL)/2;

  //##############################################
  // Averaging filters to reduce MPU random noise
  //##############################################
  // This was tried, but not used
  // GyXSum = GyXSum - GyXAvg + GyX; GyXAvg = GyXSum /AvCnt; GyX = GyXAvg;
  // GyYSum = GyYSum - GyYAvg + GyY; GyYAvg = GyYSum /AvCnt; GyY = GyYAvg;
  // GyZSum = GyZSum - GyZAvg + GyZ; GyZAvg = GyZSum /AvCnt; GyZ = GyZAvg;
}

// -------------------------------------------------------------------------------

void Pitch_PID() {
  // Calculate Pitch PID gains, called every 4ms (250Hz) once SafeMode >= 3
  // The controller is angle driver, relative to a setpoint
  // The self-balance setpoint is determined through subsequence calculations
  // so as to self-determine the best balance setpoint.
  // ############################################################################
  // Read Pitch accelerometer
  // ############################################################################
  Wire.beginTransmission(MPU_address);
  Wire.write(0x3F);                     // starting with register 0x3F (ACCEL_ZOUT_H)
  I2CErr = Wire.endTransmission();      // End the transmission with the MPU.
  if (I2CErr != 0) {return;}            // abort on errors
  delayMicroseconds(I2Cdel);            // Allow MPU time to respond

  Wire.requestFrom(MPU_address,2);      // request 4 register read
  AcZi = Wire.read()<<8|Wire.read();    // 0x3F (ACCEL_ZOUT_H) & 0x40 (ACCEL_ZOUT_L)
  AcZi -= AccOffZ;                      // add in AcZOff determined previously
  // test for rapid overshoot due to stepping around zero angle point
  AcZd = abs(AcZ - AcZi);
  if (AcZ < AcZi) {
    if (AcZd > AcRamp) {AcZ += AcRamp;} else {AcZ += AcZd;}
  }
  else if (AcZ > AcZi) {
    if (AcZd > AcRamp) {AcZ -= AcRamp;} else {AcZ -= AcZd;}
  }
  // limit values to avoid trig function problems
  if (AcZ > 8192) {AcZ = 8192;} else if (AcZ < -8192) {AcZ = -8192;}
  PtchAccAng = asin((float)AcZ/8192.0)* 57.296;

  // ############################################################################
  // Read Pitch gyro
  // ############################################################################
  Wire.beginTransmission(MPU_address);
  Wire.write(0x45);                     // starting with register 0x43 (GYRO_YOUT_H)
  I2CErr = Wire.endTransmission();      // End the transmission with the MPU.
  if (I2CErr != 0) {return;}            // abort on errors
  delayMicroseconds(I2Cdel);                // Allow MPU time to respond

  Wire.requestFrom(MPU_address,2);      // request 6 register reads
  GyY = Wire.read()<<8|Wire.read();     // 0x45 (GYRO_YOUT_H) & 0x46 (GYRO_YOUT_L)
  // get measurement timing
  GyTD = micros() - GyT;                // get the differential time between readings
  GyT = GyTD + GyT;                     // set the timer = micros(), for the next reading
  // apply calibration offsets to these raw values
  GyY -= GyrOffY;  // note GyrOffY should be zero initially in order to determin its value

  // ############################################################################
  // Calc combined angle
  // ############################################################################
  // calculate gyro angles, which will be used as the true angles
  PtchGyrAngLast = PtchGyrAng;                          // record previous angle for filtering
  float zDiv = (float)GyTD/131068000.0;                 // nominally 0.0007629 at 10ms
  PtchGyrAng += (float)GyY * zDiv;                      // Calculate the angle travelled during this 10ms loop and add this to the angle_gyro variable
  if (PtchGyrAng > 180.0) {PtchGyrAng = 180.0;}         // limit +ve gyro swing
  else if (PtchGyrAng < -180.0) {PtchGyrAng = -180.0;}  // limit -ve gyro swing
  // eliminate drift using a small amount of AccAng
  PtchGyrAng = (PtchGyrAng * 0.9980) + (PtchAccAng * 0.0020);
  // check for corrupt angles and count the errors
  if (isnan(PtchGyrAng)) {PtchGyrAng = PtchAccAng; PtchNan ++;}

  // ############################################################################
  // Lowpass filter
  // ############################################################################
  // apply lowpass FIR filter to the Gyro angle output
  // when LpfC == 1.0 zero filtering, as LpfC >> 0.0 then max filtering
  PtchGyrAng = (LpFc * PtchGyrAng) + ((1.0 - LpFc) * PtchGyrAngLast);

  // ############################################################################
  // Pitch PID gains
  // ############################################################################
  // Determine the error based on the pitch angle and setpoints
  PtchErr = PtchGyrAng - SbSp - PtchSp;


  // ############################################################################
  // PID forward gain (Brake)
  // ############################################################################
  // We close the loop around the controller once Ptch_Op > Ptch_OpBrake.
  // This effectively limits the gain of the controller giving a braking effect
  if ((Ptch_Op > Ptch_OpBrake) || (Ptch_Op < -Ptch_OpBrake)) {PtchErr += (Ptch_Op * Ptch_Brake);}

  Ptch_P = Pwr * Ptch_p_gain * PtchErr;          // calculate the P component
  Ptch_Op = Ptch_P;

  if (Ptch_i_En > 0) {
    // since the PID loop is running at 250Hz the i-gain errors build up
    // quite quickly.
    Ptch_i_mem += Pwr * Ptch_i_gain * PtchErr;
        if (Ptch_i_mem > Ptch_i_max) {Ptch_i_mem = Ptch_i_max;}
    else if (Ptch_i_mem < -Ptch_i_max) {Ptch_i_mem = -Ptch_i_max;}
    Ptch_Op += Ptch_i_mem;
  } else {Ptch_i_mem = 0.0;}

  if (Ptch_d_En > 0) {
    Ptch_D = Pwr * Ptch_d_gain * (PtchErr - PtchLastDErr);
    Ptch_Op += Ptch_D;
  } else {Ptch_D = 0.0;}
  PtchLastDErr = PtchErr;

  // ############################################################################
  // Limit PID output
  // ############################################################################
      if (Ptch_Op > Ptch_out_max) {Ptch_Op = Ptch_out_max;}
  else if (Ptch_Op < -Ptch_out_max) {Ptch_Op = -Ptch_out_max;}

  // ############################################################################
  // Apply a dead-band to PID output
  // ############################################################################
  // At small error angles, around the balance point, we want the stepper motors
  // to stop. This is achieved by setting the PID output to zero.
  if (OpMode == 0) {
    if ((Ptch_Op < Ptch_Db) && (Ptch_Op > -Ptch_Db)) {Ptch_Op = 0.0;}
  }

  // ############################################################################
  // Pitch automatic self-balance adjustment
  // ############################################################################
  // When enabled, at start-up or when there is not Nunchuk Y-demand, this function
  // adjusts the self-balance setpoint, to achieve balance.
  // The SbPtchTrend variable ensures that this is not wound up by pushing
  if (Sb && (PtchSp == 0.000)) {
         if (Ptch_Op < 0.00) {SbSp += SbSpInc;}  // Increase the self_balance_pid_setpoint if the robot is still moving forewards
    else if (Ptch_Op > 0.00) {SbSp -= SbSpInc;}  // Decrease the self_balance_pid_setpoint if the robot is still moving backwards
  }

  // ############################################################################
  // Controller calculation
  // ############################################################################
  // The following calculations are made for turning and forward/reverse motion.
  // Note that the Ptch_Op goes -ve whern driving forward, and +ve in reverse.
  Turning = 0;
  LftPtch_Op = Ptch_Op + Drive_Op;
  RhtPtch_Op = Ptch_Op + Drive_Op;

  if (RxByte & B00000100){          // If the third bit of the receive byte is set change the left and right variable to turn the robot to the right
    // Joystick pressed forward +Y
    moveDir = 1;
    if (PtchSp > -PtchSpTgt) {PtchSp -= 0.05;}          // Slowly change the setpoint angle so the robot starts leaning forewards
    if (Drive_Op > -Drive_Max) {Drive_Op -= 0.1;}
    else {Drive_Op += 0.01;}
  }
  
  else if (RxByte & B00001000){     // If the forth bit of the receive byte is set change the left and right variable to turn the robot to the right
    // Joystick pressed backward -Y
    moveDir = -1;
    if(PtchSp < PtchSpTgt) {PtchSp += 0.05;}            // Slowly change the setpoint angle so the robot starts leaning backwards
    if (Drive_Op < Drive_Max) {Drive_Op += 0.1;}
    else {Drive_Op -= 0.01;}
  }

  if (RxByte & B00000001){            // If the first bit of the receive byte is set change the left and right variable to turn the robot to the left
    // X Joystick pressed right
    Turning = 1; if (TurnSpd < TurnSpdMax) {TurnSpd += TurnSpdInc;}
    if (moveDir >= 0) {
      LftPtch_Op -= TurnSpd;          // Increase the left motor speed, more -ve
      RhtPtch_Op += TurnSpd;          // Decrease the right motor speed, more +ve
    } else {
      // steering is reverse when driving backwards
      LftPtch_Op += TurnSpd;          // Increase the left motor speed, more -ve
      RhtPtch_Op -= TurnSpd;          // Decrease the right motor speed, more +ve
    }
  }

  else if (RxByte & B00000010){       // If the second bit of the receive byte is set change the left and right variable to turn the robot to the right
    // X Joystick pressed Left
    Turning = -1; if (TurnSpd < TurnSpdMax) {TurnSpd += TurnSpdInc;}
    if (moveDir >= 0) {
      LftPtch_Op += TurnSpd;          // Decrease the left motor speed, more +ve
      RhtPtch_Op -= TurnSpd;          // Increase the right motor speed, more -ve
    } else {
      // steering is reverse when driving backwards
      LftPtch_Op -= TurnSpd;          // Decrease the left motor speed, more +ve
      RhtPtch_Op += TurnSpd;          // Increase the right motor speed, more -ve
    }
  }
  else {TurnSpd = TurnSpdMin;}      // reset steering value to minimum

  // No Y demand
  if (!(RxByte & B00001100)){
    // braking function, if no JoyY demand
    // slowly move the PID setpoint angle back to the zero position
    if (PtchSp > 0.5) {
      // If the PID setpoint is larger than 0.5 reduce the setpoint with 0.05 every loop
      PtchSp -=0.05; Drive_Op -= 0.1;
    } else if (PtchSp < -0.5) {
      // If the PID setpoint is smaller than -0.5 increase the setpoint with 0.05 every loop
      PtchSp +=0.05; Drive_Op += 0.1;
    } else {
      // If the PID setpoint is smaller than +/-0.5 set it to 0
      PtchSp = 0.000; Drive_Op = 0.0;
      if (moveDir != 0) {
        // previously moving so reset movement counters
        moveDir = 0;
      }
    }                             // reset the drive component
  }

  // ############################################################################
  // Motor pulse calculations
  // ############################################################################
  // To compensate for the non-linear behaviour of the stepper motors the following
  // calculations are needed to get an improved linear speed behaviour.
  if (OpMode == 0) {
    // ############################################################################
    // Hard deadband calculations
    // ############################################################################
    // This is based on the Brokking method, having applied a hard dead-band earlier.
    if (LftPtch_Op > 0.000) {LftPtch_Op = 405.0 - ((1.0/(LftPtch_Op + 9.0)) * 5500.0);}
    else if (LftPtch_Op < 0.000) {LftPtch_Op = -405.0 - ((1.0/(LftPtch_Op - 9.0)) * 5500.0);}

    if (RhtPtch_Op > 0.000) {RhtPtch_Op = 405.0 - ((1.0/(RhtPtch_Op + 9.0)) * 5500.0);}
    else if (RhtPtch_Op < 0.000) {RhtPtch_Op = -405.0 - (1.0/(RhtPtch_Op - 9.0)) * 5500.0;}

    // Calculate the needed pulse time for the left and right stepper motor controllers
    if (LftPtch_Op > 0.000) {Lft_Motor = (int)(400.0 - LftPtch_Op);}
    else if (LftPtch_Op < 0.000) {Lft_Motor = (int)(-400.0 - LftPtch_Op);}
    else Lft_Motor = 0.0; // stop motor steps

    if(RhtPtch_Op > 0.000) {Rht_Motor = (int)(400.0 - RhtPtch_Op);}
    else if(RhtPtch_Op < 0.000) {Rht_Motor = (int)(-400.0 - RhtPtch_Op);}
    else Rht_Motor = 0.0; // stop motor steps
  } else if (OpMode == 1) {
    // ############################################################################
    // Soft deadband calculations (10.4 Hz)
    // ############################################################################
    // This method applies a smoothed dead-band
    if (LftPtch_Op > 0.000) {
      if (LftPtch_Op > 5.000) {LftPtch_Op = 405.0 - ((1.0/(LftPtch_Op + 9.0)) * 5500.0);}
      else {LftPtch_Op *= 2.438;}
    } else if (LftPtch_Op < 0.000) {
      if (LftPtch_Op < -5.000) {LftPtch_Op = -405.0 - ((1.0/(LftPtch_Op - 9.0)) * 5500.0);}
      else {LftPtch_Op *= 2.438;}
    }

    if (RhtPtch_Op > 0.000) {
      if (RhtPtch_Op > 5.000) {RhtPtch_Op = 405.0 - ((1.0/(RhtPtch_Op + 9.0)) * 5500.0);}
      else {RhtPtch_Op *= 2.438;}
    } else if (RhtPtch_Op < 0.000) {
      if (RhtPtch_Op < -5.000) {RhtPtch_Op = -405.0 - (1.0/(RhtPtch_Op - 9.0)) * 5500.0;}
      else {RhtPtch_Op *= 2.438;}
    }

    // Calculate the needed pulse time for the left and right stepper motor controllers
    if (LftPtch_Op > 0.000) {
      if (LftPtch_Op > 12.19) {Lft_Motor = (int)(400.0 - LftPtch_Op);}
      else {Lft_Motor = (int)(2400.0 - (LftPtch_Op * 165.8));}
    } else if (LftPtch_Op < 0.000) {
      if (LftPtch_Op < -12.19) {Lft_Motor = (int)(-400.0 - LftPtch_Op);}
      else {Lft_Motor = (int)(-2400.0 - (LftPtch_Op * 165.8));}
    } else {Lft_Motor = 2400;} // minimum motor steps

    if (RhtPtch_Op > 0.000) {
      if (RhtPtch_Op > 12.19) {Rht_Motor = (int)(400.0 - RhtPtch_Op);}
      else {Rht_Motor = (int)(2400.0 - (RhtPtch_Op * 165.8));}
    } else if (RhtPtch_Op < 0.000) {
      if (RhtPtch_Op < -12.19) {Rht_Motor = (int)(-400.0 - RhtPtch_Op);}
      else {Rht_Motor = (int)(-2400.0 - (RhtPtch_Op * 165.8));}
    } else {Rht_Motor = 2400;} // minimum motor steps
  }

  // Here we pass the motor demands to the throttles for the Isr to handle.
  // As motors are driven in oposition, we reverse the direction of one of them.
  if (Lft_Motor > 0.000) {Lft_Dir = HIGH; Lft_Step_Inc= -1;} else {Lft_Dir = LOW; Lft_Step_Inc=  1;} 
  if (Rht_Motor < 0.000) {Rht_Dir = HIGH; Rht_Step_Inc=  1;} else {Rht_Dir = LOW; Rht_Step_Inc= -1;}
  digitalWrite(StepLftDir,Lft_Dir); digitalWrite(StepRhtDir,Rht_Dir);
  Lft_Motor_Throttle = Lft_Motor; Rht_Motor_Throttle = Rht_Motor;
  
  // ############################################################################
  // CZ button checks
  // ############################################################################
  // The following controller checks are performed after adjusting the motor speed.
  // Timings are all multiples of 4ms, and a short button press is < 100 counts.
  if (RxByte & B00010000){  // Z-button is pressed
    Z_Dn++;
  } else if (Z_Dn > 0) {
    // Button Z has just been released, so check timing.
    // Display_Text_2x16("Z-Button",String(Z_Dn),25);  // display the down time
    if ((Z_Dn < 100) && (Gear > 1)) {
      // short press on Z-button calls for a speed reduction
      Gear--; Set_Speeds(Gear);
      Display_Text_2x16("Gear",String(Gear),25);  // display the new speed value
      set_LED0_New(-4);    // display Gear as a binary number
    } Z_Dn = 0;
  }

  if (RxByte & B00100000){  // C-button is pressed
    C_Dn++;
    if (Z_Dn > 0) {
      // Z-button held in, so fire the guns.
      // Repeat fire if C-button is held in.
      if (GunFire == 0) {GunFire = 1;}
    }
  } else if (C_Dn > 0) {
    // C button has just been released, so check timing
    // Display_Text_2x16("C-Button",String(C_Dn),25);  // display the down time
    // If Z-button was held, in then we fire the guns, otherwise change speed
    if ((C_Dn < 100) && (Gear < 4) && (Z_Dn == 0)) {
      // short press so change gear if not at highest value
      Gear++; Set_Speeds(Gear);
      Display_Text_2x16("Gear",String(Gear),25);  // display the new speed value
      set_LED0_New(-4);    // display Gear as a binary number
    }
    C_Dn = 0;
  }
}

// -------------------------------------------------------------------------------

void Safe_Mode() {
  // Called every 4ms in either DEMO or TEST mode to
  // control the use of the PID controller, by monitoring conditions, as well
  // as a state pointer for other background functions like LED lamps
  // States:
  // -1 - re-entry state, ensure PID controller and motor drive are off
  //  0 - default TEST state, waitng to be laid down on its back
  //  1 - DEMO state, monitors angle for ready state
  //  2 - ready state, looks for near set-point start
  //  3 - setpoint triggered, so ramp up power, off we go!
  //  4 - balancing state, check for topple over
  if (!Ready) {return;}

  switch (SafeMode) {
    case -1: // re-entry state, ensure PID controller and motor drive are off
      Motors_OFF();           // turn off stepper drivers to prevent over-heating
      gotoTestMode();         // switch to TEST mode by default
      DispMode = 1;           // the type of display for a given mode
      SafeMode++; SafeCnt = 0; break;
    case 0: // default TEST state, waiting for action
      // remain in this state until the robot is placed back down for 2 sec
      // then goto state 2 and DEMO mode, if not on USB power
      if (PtchGyrAng >= 70.0) {
        // lying down at the right angle
        MyStatus = "Lying on back";
        set_LED0_New(3);
        if (SafeCnt < SafeCntMax) {SafeCnt++;}
        else {if (!USB) {SafeMode++; gotoDemoMode(); LED1_Del = 0;} else {SafeCnt = (SafeCntMax * 7)/9;}}
        // ensure that the robot is at rest
        if (fabs(PtchGyrAng - PtchGyrAngLast) > 3.0) {SafeCnt = 0;}
        PtchGyrAngLast = PtchGyrAng;
      } else {
        if (SafeCnt > 0) {SafeCnt = 0; set_LED0_New(2);}
        else {
          // Check to see if the LTOF range finder has detected a target?
          // If it has switch to LED ranging, otherwise battery monitoring
          if (RangeEn && (Range <= 160)) {
            // Target within range, so switch LED mode if needed
            if (LED0_Task != 5) {set_LED0_New(5); set_LED1_New(0);}
          } else {
            // No target detected, so remain in battery monitoring
            if (LED0_Task != 2) {set_LED0_New(2); set_LED1_New(1);}
          }
        }
             if ((PtchGyrAng < 2.0) && (PtchGyrAng > -2.0)) {MyStatus = "Upright";}
        else if (PtchGyrAng >= 2.0) {MyStatus = "Leaning back";}
        else if (PtchGyrAng <= -70.0) {MyStatus = "Lying on front";}
        else if (PtchGyrAng <= -2.0) {MyStatus = "Leaning forward";}
      } break;
    case 1: // Monitor angle for ready state.
      // The robot has to be raised from the the lying down state
      // towards the vertical position, at 20 degrees we turn the motors on
      MyStatus = "Ready";
      if (fabs(PtchGyrAng) <= 20.0) {SafeMode++; SafeCnt = 0; PtchGyrAng = PtchAccAng;}
      break;
    case 2: // Ready state, looks for near set-point start.
      // Check that we remain within the 'ready' region, but if we aren't,
      // then drop back into state 1. Allow a margin to create a hysteresis effect.
      MyStatus = "Powering up";
      if (fabs(PtchGyrAng) > 25.0) {SafeMode = 1; Motors_OFF(); return;}  // drop back to state 1
      // now look for centre point
      else if ((PtchGyrAng <= (PtchStartAng + 0.3)) && (PtchGyrAng >= (PtchStartAng - 0.3))) {
        // robot must be in trigger zone for 40ms before going active
        SafeCnt++;
        if (SafeCnt >= 10) {
          // switch to power-on mode
          Drive_Op = 0.0;             // zero drive value
          PtchGyrAng = PtchAccAng;    // set gyro angle to match AcY angle
          Ptch_Op = 0.0;              // reset PID output for 1st pass
          moveDir = 0;                // reset move direction flag. 0 == stationary
          PtchSp = 0.0;               // reset any demands on the primary setpoint
          // Ptch_i_En = 0;              // disable integrator initially
          Ptch_i_En = 1;              // enable the integrator
          Ptch_i_mem = 0;             // zero the i-gain memory
          // Ptch_d_En = 0;              // disable d-gain initially
          Ptch_d_En = 1;              // enable d-gain initially
          PtchLastDErr = 0.0;         // zero the last d-gain
          RxByte = 0b00000000;        // clear the byte mask
          // Sb = true;                  // enable self-balance algorithm
          Sb = false;                 // hold off self-balance algorithm
          SbSp = PtchStartAng;        // set Pitch self-balancing set point to an initial angle
          // Sb_trend = 0;               // counter used in auto self-balance point adjustment
          PtchNan = 0;                // PItch 'Not A Number' error count
          Pwr = 0.1;                  // limit PID out to 10% initially
          Lft_Motor_Throttle = 0;     // stop left motor if turning
          Rht_Motor_Throttle = 0;     // stop right motor if turning
          SafeMode++;                 // switch to power-up SafeMode 3
          MyStatus = "Active";
        }
      } else {SafeCnt = 0;}
      // in the ready region we slowly drive the wheel towards the balance point
      if (PtchGyrAng > (PtchStartAng + 0.3)) {
        // driver slowly backwards towards balance point
        Lft_Motor_Throttle = 4200/(1 + (int)PtchGyrAng); Rht_Motor_Throttle = Lft_Motor_Throttle;
        if (!IsrRun) {
          Lft_Dir = HIGH; digitalWrite(StepLftDir,Lft_Dir);
          Rht_Dir =  LOW; digitalWrite(StepRhtDir,Rht_Dir);
          // Lft_Motor_Throttle = 200; Rht_Motor_Throttle = 200;
          Lft_Motor_Tgt = 0; Rht_Motor_Tgt = 0;
          Lft_En= LOW; digitalWrite(StepLftEn,Lft_En);
          Rht_En= LOW; digitalWrite(StepRhtEn,Rht_En);
          IsrRun = true;
        } else {
          if (!Lft_Dir) {Lft_Dir = HIGH; digitalWrite(StepLftDir,Lft_Dir);}
          if ( Rht_Dir) {Rht_Dir =  LOW; digitalWrite(StepRhtDir,Rht_Dir);}
        }
      } else if (PtchGyrAng < (PtchStartAng - 0.3)) {
        // driver slowly forwards towards balance point
        Lft_Motor_Throttle = 4200/(1 - (int)PtchGyrAng); Rht_Motor_Throttle = Lft_Motor_Throttle;
        if (!IsrRun) {
          Lft_Dir =  LOW; digitalWrite(StepLftDir,Lft_Dir);
          Rht_Dir = HIGH; digitalWrite(StepRhtDir,Rht_Dir);
          // Lft_Motor_Throttle = 200; Rht_Motor_Throttle = 200;
          Lft_En= LOW; digitalWrite(StepLftEn,Lft_En);
          Rht_En= LOW; digitalWrite(StepRhtEn,Rht_En);
          Lft_Motor_Tgt = 0; Rht_Motor_Tgt = 0;
          IsrRun = true;
        } else {
          if ( Lft_Dir) {Lft_Dir =  LOW; digitalWrite(StepLftDir,Lft_Dir);}
          if (!Rht_Dir) {Rht_Dir = HIGH; digitalWrite(StepRhtDir,Rht_Dir);}
        }
      } else {Motors_OFF();}  // STOP motors in centre deadband
      break;
    case 3: // setpoint triggered, so ramp up power and transition to RollSafeMode = 4
      // at 4ms cycles, this takes 400ms from 50%
      if (Pwr < 1.00) {Pwr+= 0.01;} else {
        SafeMode++;
        Sb = true;                  // enable self-balance algorithm
        // Ptch_i_En = 1;
        // Ptch_d_En = 1;
      } break;
    case 4: // balancing state
      // Test for robot falling over
      if (fabs(PtchAccAng) >= toppleAng) {
        // toppling over, so abort and reset safe mode
        Motors_OFF();           // turn off stepper drivers to prevent over-heating
        SafeMode = -1;          // return to looking for 'ready' state
        BackAwayTask = 0;       // Cancel Back Away functions.
        set_LED1_New(1);        // set LEDs to Demo mode
        Display_Topple();       // display event
        DispMode = 1;           // return to battery display
        if (RangeEn) {VL53L1X_OFF();} // turn of range finder
        MyStatus = "Safe";
      }
      // Control LED tasks whilst balancing
      // Check to see if the LTOF range finder has detected a target?
      // If it has switch to LED ranging, otherwise battery monitoring
      if (RangeEn && (Range <= 160)) {
        // Range finder is enabled
        // Target within range, so switch LED mode if needed
        if (LED0_Task != 5) {set_LED0_New(5); set_LED1_New(0);}
      } else {
        // No target detected, so remain in battery monitoring
        if (LED0_Task != 2) {set_LED0_New(2); set_LED1_New(1);}
      }
      break;
  }
}

// -------------------------------------------------------------------------------

