///////////////////////////////////////////////////////////////////////////////////
//
// Functions mainly used for development and debugging
//
///////////////////////////////////////////////////////////////////////////////////

void ScopeReport(int zGPnn ) {
  ///////////////////////////////////////////////////////////////////////
  // a choice of print statements to inspect variables dynamically
  ///////////////////////////////////////////////////////////////////////
  // Note at 4ms cycle and 115200 baud string length is limited to 46 chars
  // max in a 4ms period, otherwise blocking will occur. If used over WiFi then
  // link corruption may occur; so to prevent rogue values we send data twice and
  // checked on receive as a valid pair of duplicate values, otherwise disgard it.
  // Adjust the following three values to match the switch() list below, so that
  // title reports the extent of the GPnn list to the Monitor+ app, whenever a
  // new GPnn us called for.
  GPmin = 0; GPmax = 7; String zGM = "GM7\n";

  switch (zGPnn) {
    case 0: // Custom GP(0) mode
      // in in this mode the user can click on the title elements and select them
      // using a set of four pointers, SCpPnt0 - 3
      if (GP_Title) {
        PrintTx += "#";
        Scope_Table(ScpPnt0); PrintTx += ScpTitle$ + ",";
        Scope_Table(ScpPnt1); PrintTx += ScpTitle$ + ",";
        Scope_Table(ScpPnt2); PrintTx += ScpTitle$ + ",";
        Scope_Table(ScpPnt3); PrintTx += ScpTitle$ + "\n" + zGM;
        GP_Title = false;}
      Scope_Table(ScpPnt0); PrintTx += ScpVal$ + ",";
      Scope_Table(ScpPnt1); PrintTx += ScpVal$ + ",";
      Scope_Table(ScpPnt2); PrintTx += ScpVal$ + ",";
      Scope_Table(ScpPnt3); PrintTx += ScpVal$ + "\n";
      break;

    case 1:
      if (GP_Title) {PrintTx += "#AcX,AcY,AcZ\n" + zGM; GP_Title = false;}
      PrintTx += String(AcX) + ",";
      PrintTx += String(AcY) + ",";
      PrintTx += String(AcZ) + "\n";
      break;

    case 2:
      if (GP_Title) {PrintTx += "#GyX,GyY,GyZ\n" + zGM; GP_Title = false;}
      PrintTx += String(GyX) + ",";
      PrintTx += String(GyY) + ",";
      PrintTx += String(GyZ) + "\n";
      break;

    case 3:
      if (GP_Title) {PrintTx += "#RangeRaw,Range,RangeMin,RangeMax\n" + zGM; GP_Title = false;}
      PrintTx += String(RangeRaw) + ",";
      PrintTx += String(Range) + ",";
      PrintTx += String(RangeMin) + ",";
      PrintTx += String(RangeMax) + "\n";
      break;

    case 4:
      if (GP_Title) {PrintTx += "#RangeRaw,Range,RangeTrkHi,RangeTrkLo\n" + zGM; GP_Title = false;}
      PrintTx += String(RangeRaw) + ",";
      PrintTx += String(Range) + ",";
      PrintTx += String(RangeTrkHi) + ",";
      PrintTx += String(RangeTrkLo) + "\n";
      break;

    case 5:
      if (GP_Title) {PrintTx += "#Ptch_Op,Ptch_i_mem,Ptch_D,Ptch_P\n" + zGM; GP_Title = false;}
      PrintTx += String(Ptch_Op) + ",";
      PrintTx += String(Ptch_i_mem) + ",";
      PrintTx += String(Ptch_D) + ",";
      PrintTx += String(Ptch_P) + "\n";
      break;

    case 6:
      if (GP_Title) {PrintTx += "#LftPtch_Op,RhtPtch_Op,Lft_Motor_Throttle,Rht_Motor_Throttle\n" + zGM; GP_Title = false;}
      PrintTx += String(LftPtch_Op) + ",";
      PrintTx += String(RhtPtch_Op) + ",";
      PrintTx += String(Lft_Motor_Throttle) + ",";
      PrintTx += String(Rht_Motor_Throttle) + "\n";
      break;

    case 7:
      if (GP_Title) {PrintTx += "#PtchAccAng,PtchGyrAng,0\n" + zGM; GP_Title = false;}
      PrintTx += String(PtchAccAng) + ",";
      PrintTx += String(PtchGyrAng) + ",";
      PrintTx += "0\n";
      break;

    default:
      // zRPnn was out of range so reset the reporting pointer and send commands
      // to set RP = 0 and RMax to the highest number in your case list. ie. 7
      GPnn = GPmin; PrintTx += "GP" + String(GPmin) + "\n";
      PrintTx += "GM" + String(GPmax) + "\n"; break;
  }
}

// --------------------------------------------------------------------------------

void Scope_Table(int16_t zPnt) {
  // called to select a variable from a list, in custom Scope GP0 mode
  // you can add more variables to the table if you want, but remember
  // to renumber the cases and set ScpDpth appropriately
  switch(zPnt) {
    case  0: ScpVal$ = "0"; ScpTitle$ = "0"; break;
    case  1: ScpVal$ = String(AcX); ScpTitle$ = "AcX"; break;
    case  2: ScpVal$ = String(AcY); ScpTitle$ = "AcY"; break;
    case  3: ScpVal$ = String(AcZ); ScpTitle$ = "AcZ"; break;

    case  4: ScpVal$ = String(GyX); ScpTitle$ = "GyX"; break;
    case  5: ScpVal$ = String(GyY); ScpTitle$ = "GyY"; break;
    case  6: ScpVal$ = String(GyZ); ScpTitle$ = "GyZ"; break;

    case  7: ScpVal$ = String(Lft_Motor); ScpTitle$ = "Lft_Motor"; break;
    case  8: ScpVal$ = String(Lft_Motor_Throttle); ScpTitle$ = "Lft_Motor_Throttle"; break;
    case  9: ScpVal$ = String(LftPtch_Op); ScpTitle$ = "LftPtch_Op"; break;

    case 10: ScpVal$ = String(PtchAccAng); ScpTitle$ = "PtchAccAng"; break;
    case 11: ScpVal$ = String(Ptch_D); ScpTitle$ = "Ptch_D"; break;
    case 12: ScpVal$ = String(PtchErr); ScpTitle$ = "PtchErr"; break;
    case 13: ScpVal$ = String(PtchGyrAng); ScpTitle$ = "PtchGyrAng"; break;
    case 14: ScpVal$ = String(Ptch_i_mem); ScpTitle$ = "Ptch_i_mem"; break;
    case 15: ScpVal$ = String(Ptch_Op); ScpTitle$ = "Ptch_Op"; break;
    case 16: ScpVal$ = String(Ptch_P); ScpTitle$ = "Ptch_P"; break;
    case 17: ScpVal$ = String(Pwr); ScpTitle$ = "Pwr"; break;

    case 18: ScpVal$ = String(RangeRaw); ScpTitle$ = "RangeRaw"; break;
    case 19: ScpVal$ = String(Range); ScpTitle$ = "Range"; break;
    case 20: ScpVal$ = String(RangeMin); ScpTitle$ = "RangeMin"; break;
    case 21: ScpVal$ = String(RangeMax); ScpTitle$ = "RangeMax"; break;
    case 22: ScpVal$ = String(Rht_Motor); ScpTitle$ = "Rht_Motor"; break;
    case 23: ScpVal$ = String(Rht_Motor_Throttle); ScpTitle$ = "Rht_Motor_Throttle"; break;
    case 24: ScpVal$ = String(RhtPtch_Op); ScpTitle$ = "RhtPtch_Op"; break;
    
    case 25: ScpVal$ = String(SafeMode); ScpTitle$ = "SafeMode"; break;
    case 26: ScpVal$ = String(SbSp); ScpTitle$ = "SbSp"; break;
  }
  // set ScpDpth = last item in the table
  ScpDpth = 25;
}

// --------------------------------------------------------------------------------

